@@ -2,6 +2,7 @@
 # Copyright (c) 2019 VMware, Inc. All Rights Reserved.
 # SPDX-License-Identifier: BSD-2-Clause
 
+import pyvcloud.vcd.exceptions as vcd_exceptions
 import requests
 
 from container_service_extension.minor_error_codes import MinorErrorCode
@@ -32,6 +33,25 @@ class CseServerError(Exception):
     """Base class for cse server side exceptions."""
 
 
+class CseUpgradeError(Exception):
+    """Base class for cse server upgrade exception."""
+
+
+class CseServerNotRunningError(CseServerError):
+    """Raised when CLI makes requests to CSE server when it is not available."""  # noqa: E501
+
+    def __init__(self, msg=None):
+        self.msg = msg
+
+    def __str__(self):
+        if self.msg:
+            return self.msg
+        return "Please contact administrator, CSE server seems to be" \
+               " down. CSE- CLI can now only be used to manage TKG " \
+               " clusters (but not native). Once CSE server is up, please" \
+               " re-login to manage both native and tkg clusters."
+
+
 class CseRequestError(CseServerError):
     """Base class for all incoming CSE REST request errors."""
 
@@ -161,8 +181,8 @@ class NodeCreationError(NodeOperationError):
             f"{self.error_message}"
 
 
-class MasterNodeCreationError(NodeOperationError):
-    """Raised when any error happens while adding master node."""
+class ControlPlaneNodeCreationError(NodeOperationError):
+    """Raised when any error happens while adding control plane node."""
 
 
 class WorkerNodeCreationError(NodeOperationError):
@@ -191,3 +211,44 @@ class PksClusterNotFoundError(PksServerError):
 
 class PksDuplicateClusterError(PksServerError):
     """Raised when multiple PKS clusters of same name detected."""
+
+
+class DefNotSupportedException(CseServerError):
+    """Raised if API version does not support Defined entity feature."""
+
+
+class DefEntityResolutionError(CseServerError):
+    """Raised if Defined entity is not resolved properly."""
+
+    def __init__(self, id: str, state: str, msg: str = None):
+        self.id = id
+        self.state = state
+        self.msg = msg
+
+    def __str__(self):
+        return f"{self.state}: Resolution of the cluster with defined entity" \
+               f" id:{self.id} failed with error message: {self.msg}"
+
+
+class GlobalPvdcComputePolicyNotSupported(vcd_exceptions.OperationNotSupportedException): # noqa: E501
+    """Raised when global pvdc compute policies are not supported."""
+
+
+class VersionCompatibilityError(CseServerError):
+    """Raised on CSE or VCD API version difference.
+
+    Raised if local CSE version or target VCD API version is different
+    than the versions already registered by CSE.
+    """
+
+
+class DefEntityServiceError(InternalServerRequestError):
+    """Raised on any defined entity service operation failure."""
+
+
+class DefSchemaServiceError(InternalServerRequestError):
+    """Raised on any of defined schema service error."""
+
+
+class MQTTExtensionError(CseServerError):
+    """Raised on any error when setting up the MQTT extension."""
