@@ -69,7 +69,7 @@ class MQTTExtensionManager:
     def __init__(self, sysadmin_client: vcd_client.Client,
                  debug_logger=logger.SERVER_LOGGER, log_wire=True):
         # Ensure correct credentials and api version
-        vcd_utils.raise_error_if_not_sysadmin(sysadmin_client)
+        vcd_utils.raise_error_if_user_not_from_system_org(sysadmin_client)
         client_api_version = float(sysadmin_client.get_api_version())
         if client_api_version < constants.MQTT_MIN_API_VERSION:
             raise ValueError(f'API version {client_api_version} '
@@ -354,7 +354,7 @@ class MQTTExtensionManager:
         }
         response_body = self._cloudapi_client.do_request(
             method=RequestMethod.POST,
-            cloudapi_version=cloudapi_constants.CLOUDAPI_VERSION_1_0_0,
+            cloudapi_version=cloudapi_constants.CloudApiVersion.VERSION_1_0_0,
             resource_url_relative_path=constants.TOKEN_PATH,
             payload=payload)
         token_info = {
@@ -377,7 +377,7 @@ class MQTTExtensionManager:
         token_ids = []
         response_body = self._cloudapi_client.do_request(
             method=RequestMethod.GET,
-            cloudapi_version=cloudapi_constants.CLOUDAPI_VERSION_1_0_0,
+            cloudapi_version=cloudapi_constants.CloudApiVersion.VERSION_1_0_0,
             resource_url_relative_path=constants.TOKEN_PATH)
         ext_combine_name = f"{ext_vendor}/{ext_name}/{ext_version}"
         for token_info in response_body['values']:
@@ -394,7 +394,7 @@ class MQTTExtensionManager:
         """
         self._cloudapi_client.do_request(
             method=RequestMethod.DELETE,
-            cloudapi_version=cloudapi_constants.CLOUDAPI_VERSION_1_0_0,
+            cloudapi_version=cloudapi_constants.CloudApiVersion.VERSION_1_0_0,
             resource_url_relative_path=f"{constants.TOKEN_PATH}/"
                                        f"{token_id}")
 
@@ -422,7 +422,7 @@ class MQTTExtensionManager:
         try:
             self._cloudapi_client.do_request(
                 method=RequestMethod.GET,
-                cloudapi_version=cloudapi_constants.CLOUDAPI_VERSION_1_0_0,
+                cloudapi_version=cloudapi_constants.CloudApiVersion.VERSION_1_0_0,  # noqa: E501
                 resource_url_relative_path=f"{constants.TOKEN_PATH}/"
                                            f"{token_id}")
         except requests.exceptions.HTTPError:
