@@ -51,9 +51,6 @@ CSE_GLOBAL_PVDC_COMPUTE_POLICY_DESCRIPTION = \
 CSE_CLUSTER_KUBECONFIG_PATH = '/root/.kube/config'
 
 # MQTT constants
-# TODO: version should be major.minor.patch
-#  Also, patch change doesn't mean version needs to be updated,
-#  but major upgrade with api changes should lead to updating the version
 ADMIN_EXT_SERVICE_PATH = 'admin/extension/service'
 API_FILTER_PATH = 'apifilter'
 API_FILTERS_PATH = 'apifilters'
@@ -66,11 +63,26 @@ MQTT_MIN_API_VERSION = 35.0
 MQTT_TOKEN_NAME = "mqttCseToken"
 TOKEN_PATH = 'tokens'
 
+
 # Encryption constants
 PBKDF2_ITERATIONS = 100000
 SALT_SIZE = 32
 PBKDF2_OUTPUT_SIZE = 32
 
+# Names of Message Consumer Thread and Watchdog Thread
+MESSAGE_CONSUMER_THREAD = 'MessageConsumer'
+WATCHDOG_THREAD = 'ConsumerWatchdog'
+
+
+# Config file error messages
+CONFIG_DECRYPTION_ERROR_MSG = \
+    "Config file decryption failed: invalid decryption password"
+VCENTER_LOGIN_ERROR_MSG = "vCenter login failed (check config file for "\
+    "vCenter username/password)."
+
+# Request Id format for logging
+REQUEST_ID_FORMAT = 'Request Id: %(requestId)s | '
+
 
 @unique
 class NodeType(str, Enum):
@@ -292,3 +304,117 @@ class ExtensionType(str, Enum):
     MQTT = 'MQTT'
     AMQP = 'AMQP'
     NONE = 'None'
+
+
+@unique
+class AclGrantType(str, Enum):
+    MembershipACLGrant = "MembershipAccessControlGrant"
+
+
+@unique
+class AclAccessLevelId(str, Enum):
+    AccessLevelReadWrite = "urn:vcloud:accessLevel:ReadWrite"
+
+
+@unique
+class AclMemberId(str, Enum):
+    SystemOrgId = "urn:vcloud:org:a93c9db9-7471-3192-8d09-a8f7eeda85f9"
+
+
+# CSE Service Role Name
+CSE_SERVICE_ROLE_NAME = 'CSE Service Role'
+CSE_SERVICE_ROLE_DESC = "CSE Service Role has all the rights necessary for \
+        CSE to operate"
+CSE_SERVICE_ROLE_RIGHTS = [
+    "Access Control List: View",
+    "Access Control List: Manage",
+    "AMQP Settings: View",
+    "Catalog: Add vApp from My Cloud",
+    "Catalog: Create / Delete a Catalog",
+    "Catalog: Edit Properties",
+    "Catalog: Publish",
+    "Catalog: Sharing",
+    "Catalog: View ACL",
+    "Catalog: View Private and Shared Catalogs",
+    "Catalog: View Published Catalogs",
+    "Content Library System Settings: View",
+    "Custom entity: Create custom entity definitions",
+    "Custom entity: Delete custom entity definitions",
+    "Custom entity: Edit custom entity definitions",
+    "Custom entity: View custom entity definitions",
+    "Extension Services: View",
+    "Extensions: View",
+    "External Service: Manage",
+    "External Service: View",
+    "General: View Error Details",
+    "Group / User: View",
+    "Host: View",
+    "Kerberos Settings: View",
+    "Organization Network: View",
+    "Organization vDC Compute Policy: Admin View",
+    "Organization vDC Compute Policy: Manage",
+    "Organization vDC Compute Policy: View",
+    "Organization vDC Kubernetes Policy: Edit",
+    "Organization vDC Network: Edit Properties",
+    "Organization vDC Network: View Properties",
+    "Organization vDC: Extended Edit",
+    "Organization vDC: Extended View",
+    "Organization vDC: View",
+    "Organization: Perform Administrator Queries",
+    "Organization: View",
+    "Provider Network: View",
+    "Provider vDC Compute Policy: Manage",
+    "Provider vDC Compute Policy: View",
+    "Provider vDC: View",
+    "Right: Manage",
+    "Right: View",
+    "Rights Bundle: View",
+    "Role: Create, Edit, Delete, or Copy",
+    "Service Configuration: Manage",
+    "Service Configuration: View",
+    "System Settings: View",
+    "Task: Resume, Abort, or Fail",
+    "Task: Update",
+    "Task: View Tasks",
+    "Token: Manage",
+    "UI Plugins: Define, Upload, Modify, Delete, Associate or Disassociate",
+    "UI Plugins: View",
+    "vApp Template / Media: Copy",
+    "vApp Template / Media: Create / Upload",
+    "vApp Template / Media: Edit",
+    "vApp Template / Media: View",
+    "vApp Template: Checkout",
+    "vApp Template: Import",
+    "vApp: Allow All Extra Config",
+    "vApp: Allow Ethernet Coalescing Extra Config",
+    "vApp: Allow Latency Extra Config",
+    "vApp: Allow Matching Extra Config",
+    "vApp: Allow NUMA Node Affinity Extra Config",
+    "vApp: Create / Reconfigure",
+    "vApp: Delete",
+    "vApp: Edit Properties",
+    "vApp: Edit VM CPU and Memory reservation settings in all VDC types",
+    "vApp: Edit VM CPU",
+    "vApp: Edit VM Compute Policy",
+    "vApp: Edit VM Hard Disk",
+    "vApp: Edit VM Memory",
+    "vApp: Edit VM Network",
+    "vApp: Edit VM Properties",
+    "vApp: Manage VM Password Settings",
+    "vApp: Power Operations",
+    "vApp: Shadow VM View",
+    "vApp: Upload",
+    "vApp: Use Console",
+    "vApp: VM Boot Options",
+    "vApp: VM Check Compliance",
+    "vApp: VM Migrate, Force Undeploy, Relocate, Consolidate",
+    "vApp: View VM and VM's Disks Encryption Status",
+    "vApp: View VM metrics",
+    "vCenter: View",
+    "vSphere Server: View",
+    "vmware:tkgcluster: Administrator Full access",
+    "vmware:tkgcluster: Administrator View",
+    "vmware:tkgcluster: Full Access",
+    "vmware:tkgcluster: Modify",
+    "vmware:tkgcluster: View"
+]
