@@ -84,18 +84,37 @@ def cse_server():
     - Stop CSE server
     """
     config = testutils.yaml_to_dict(env.BASE_CONFIG_FILEPATH)
-    install_cmd = ['install',
-                   '--config', env.ACTIVE_CONFIG_FILEPATH,
-                   '--ssh-key', env.SSH_KEY_FILEPATH,
-                   '--skip-config-decryption']
-    env.setup_active_config()
-    result = env.CLI_RUNNER.invoke(cli, install_cmd,
-                                   input='y',
-                                   catch_exceptions=False)
+    config = env.setup_active_config()
+    if env.is_cse_registered():
+        cmd = ['upgrade',
+               '--config', env.ACTIVE_CONFIG_FILEPATH,
+               '--ssh-key', env.SSH_KEY_FILEPATH,
+               '--skip-config-decryption',
+               '--skip-template-creation']
+    else:
+        cmd = ['install',
+               '--config', env.ACTIVE_CONFIG_FILEPATH,
+               '--ssh-key', env.SSH_KEY_FILEPATH,
+               '--skip-config-decryption',
+               '--skip-template-creation']
+    result = env.CLI_RUNNER.invoke(cli, cmd, input='y', catch_exceptions=False)
     assert result.exit_code == 0,\
-        testutils.format_command_info('cse', install_cmd, result.exit_code,
+        testutils.format_command_info('cse', cmd, result.exit_code,
                                       result.output)
 
+    # Create missing templates
+    for template_config in env.TEMPLATE_DEFINITIONS:
+        cmd = f"template install {template_config['name']} " \
+              f"{template_config['revision']} " \
+              f"--config {env.ACTIVE_CONFIG_FILEPATH} " \
+              f"--ssh-key {env.SSH_KEY_FILEPATH} " \
+              f"--skip-config-decryption"
+        result = env.CLI_RUNNER.invoke(
+            cli, cmd.split(), catch_exceptions=False)
+        assert result.exit_code == 0,\
+            testutils.format_command_info('cse', cmd, result.exit_code,
+                                          result.output)
+
     # start cse server as subprocess
     cmd = f"cse run -c {env.ACTIVE_CONFIG_FILEPATH} --skip-config-decryption"
     p = None
@@ -107,7 +126,7 @@ def cse_server():
                              stderr=subprocess.STDOUT)
     time.sleep(env.WAIT_INTERVAL * 3)  # server takes a little while to set up
 
-    # enable kubernetes functionality on our ovdc
+    # Enable kubernetes functionality on our ovdc
     # by default, an ovdc cannot deploy kubernetes clusters
     # TODO() this should be removed once this behavior is changed
     cmd = f"login {config['vcd']['host']} {constants.SYSTEM_ORG_NAME} " \
@@ -127,12 +146,13 @@ def cse_server():
     assert result.exit_code == 0,\
         testutils.format_command_info('vcd', cmd, result.exit_code,
                                       result.output)
-    cmd = f"cse ovdc enable {config['broker']['vdc']} -k " \
-          f"{constants.K8sProvider.NATIVE}"
-    result = env.CLI_RUNNER.invoke(vcd, cmd.split(), catch_exceptions=False)
-    assert result.exit_code == 0,\
-        testutils.format_command_info('vcd', cmd, result.exit_code,
-                                      result.output)
+    # TODO (metadata based enablement for < v35): Commenting this for now
+    # Should revisit after decision
+    # cmd = f"cse ovdc enable {config['broker']['vdc']}"
+    # result = env.CLI_RUNNER.invoke(vcd, cmd.split(), catch_exceptions=False)
+    # assert result.exit_code == 0,\
+    #     testutils.format_command_info('vcd', cmd, result.exit_code,
+    #                                   result.output)
     cmd = f"catalog acl add {config['broker']['catalog']} " \
           f"\'org:{config['broker']['org']}:ReadOnly\'"
     result = env.CLI_RUNNER.invoke(vcd, cmd.split(), catch_exceptions=False)
@@ -361,7 +381,7 @@ def test_0050_vcd_cse_system_toggle(config, test_runner_username,
     cmd_list = [
         cmd_binder(cmd=env.SYS_ADMIN_LOGIN_CMD, exit_code=0,
                    validate_output_func=None, test_user='sys_admin'),
-        cmd_binder(cmd=f"cse system disable", exit_code=0,
+        cmd_binder(cmd="cse system disable", exit_code=0,
                    validate_output_func=None, test_user='sys_admin'),
         cmd_binder(cmd=env.USERNAME_TO_LOGIN_CMD[test_runner_username],
                    exit_code=0, validate_output_func=None,
@@ -420,7 +440,7 @@ def test_0070_vcd_cse_cluster_create(config, test_runner_username):
                    validate_output_func=None, test_user=test_runner_username),
         cmd_binder(cmd=f"cse cluster create "
                        f"{env.USERNAME_TO_CLUSTER_NAME[test_runner_username]}"
-                       f" -n {config['broker']['network']} -N 1 ", exit_code=0,
+                       f" -n {config['broker']['network']} -N 0 ", exit_code=0,
                    validate_output_func=None, test_user=test_runner_username),
         cmd_binder(cmd=env.USER_LOGOUT_CMD, exit_code=0,
                    validate_output_func=None, test_user=test_runner_username)
@@ -447,7 +467,7 @@ def test_0080_vcd_cse_cluster_list(test_runner_username):
         cmd_binder(cmd=env.USERNAME_TO_LOGIN_CMD[test_runner_username],
                    exit_code=0,
                    validate_output_func=None, test_user=test_runner_username),
-        cmd_binder(cmd=f"cse cluster list", exit_code=0,
+        cmd_binder(cmd="cse cluster list", exit_code=0,
                    validate_output_func=list_cluster_output_validator,
                    test_user=test_runner_username),
         cmd_binder(cmd=env.USER_LOGOUT_CMD, exit_code=0,
@@ -506,10 +526,81 @@ def test_0100_vcd_cse_cluster_config(test_runner_username):
     print(f"Successful cluster config on {env.USERNAME_TO_CLUSTER_NAME[test_runner_username]}.")  # noqa
 
 
+def generate_validate_node_count_func(expected_nodes):
+    """Generate validator function to verify the number of nodes in the cluster.
+
+    :param expected_nodes: Expected number of nodes in the cluster
+
+    :return validator: function(output, test_user)
+    """
+    node_pattern = r'(node-\S+)'
+
+    def validator(output, test_runner_username):
+        cmd_binder = collections.namedtuple('UserCmdBinder',
+                                            'cmd exit_code validate_output_func ' # noqa: E501
+                                            'test_user')
+        print(f"Running cluster info operation for {test_runner_username}")
+        cmd_list = [
+            cmd_binder(cmd=f"cse cluster info {env.USERNAME_TO_CLUSTER_NAME[test_runner_username]}",   # noqa
+                       exit_code=0,
+                       validate_output_func=None,
+                       test_user=test_runner_username)
+        ]
+        cmd_results = execute_commands(cmd_list)
+
+        return len(re.findall(node_pattern, cmd_results[0].output)) == expected_nodes # noqa
+
+    return validator
+
+
 @pytest.mark.parametrize('test_runner_username', [env.SYS_ADMIN_NAME,
                                                   env.ORG_ADMIN_NAME,
                                                   env.VAPP_AUTHOR_NAME])
-def test_0110_vcd_cse_node_operation(test_runner_username, config):
+def test_0110_vcd_cse_cluster_resize(test_runner_username, config):
+    """Test 'vcd cse cluster resize ...' commands."""
+    node_pattern = r'(node-\S+)'
+    cmd_binder = collections.namedtuple('UserCmdBinder',
+                                        'cmd exit_code validate_output_func '
+                                        'test_user')
+
+    print(f"Running cluster info operation for {test_runner_username}")
+    cmd_list = [
+        cmd_binder(cmd=env.USERNAME_TO_LOGIN_CMD[test_runner_username],
+                   exit_code=0,
+                   validate_output_func=None, test_user=test_runner_username),
+        cmd_binder(cmd=f"org use {config['broker']['org']}", exit_code=0,
+                   validate_output_func=None, test_user=test_runner_username),
+        cmd_binder(cmd=f"vdc use {config['broker']['vdc']}", exit_code=0,
+                   validate_output_func=None, test_user=test_runner_username),
+        cmd_binder(cmd=f"cse cluster info {env.USERNAME_TO_CLUSTER_NAME[test_runner_username]}",   # noqa
+                   exit_code=0,
+                   validate_output_func=None,
+                   test_user=test_runner_username)
+    ]
+    cmd_results = execute_commands(cmd_list)
+
+    num_nodes = len(re.findall(node_pattern, cmd_results[-1].output))
+
+    print(f"Running cluster resize operation for {test_runner_username}")
+
+    cmd_list = [
+        cmd_binder(cmd=f"cse cluster resize -N {num_nodes+1} -n {config['broker']['network']}"  # noqa
+                       f" {env.USERNAME_TO_CLUSTER_NAME[test_runner_username]}", # noqa: E501
+                   exit_code=0, validate_output_func=generate_validate_node_count_func(num_nodes+1), # noqa
+                   test_user=test_runner_username),
+        cmd_binder(cmd=f"cse cluster resize -N 0 -n {config['broker']['network']}" # noqa
+                       f" {env.USERNAME_TO_CLUSTER_NAME[test_runner_username]}", # noqa: E501
+                   exit_code=0, validate_output_func=generate_validate_node_count_func(0), # noqa
+                   test_user=test_runner_username)
+    ]
+    execute_commands(cmd_list)
+    print(f"Successful cluster resize on {env.USERNAME_TO_CLUSTER_NAME[test_runner_username]}.") # noqa
+
+
+@pytest.mark.parametrize('test_runner_username', [env.SYS_ADMIN_NAME,
+                                                  env.ORG_ADMIN_NAME,
+                                                  env.VAPP_AUTHOR_NAME])
+def test_0120_vcd_cse_node_operation(test_runner_username, config):
     """Test 'vcd cse node create/list/info/delete ...' commands.
 
     Test node creation from different persona's- sys_admin, org_admin
@@ -521,7 +612,7 @@ def test_0110_vcd_cse_node_operation(test_runner_username, config):
     different users
     """
     node_pattern = r'(node-\S+)'
-    num_nodes = 1
+    num_nodes = 0 # last resize scaled the cluster down to 0 nodes
     cmd_binder = collections.namedtuple('UserCmdBinder',
                                         'cmd exit_code validate_output_func '
                                         'test_user')
@@ -612,7 +703,7 @@ def test_0130_vcd_cse_cluster_delete(config):
     cmd_binder = collections.namedtuple('UserCmdBinder',
                                         'cmd exit_code validate_output_func '
                                         'test_user')
-    print(f"Running cluster delete operations")
+    print("Running cluster delete operations")
     cmd_list = [
         cmd_binder(cmd=env.VAPP_AUTHOR_LOGIN_CMD,
                    exit_code=0,
@@ -653,7 +744,7 @@ def test_0130_vcd_cse_cluster_delete(config):
     for cluster_name in env.USERNAME_TO_CLUSTER_NAME.values():
         assert not env.vapp_exists(cluster_name), \
             f"Cluster {cluster_name} exists when it should not"
-    print(f"Successfully deleted clusters.")
+    print("Successfully deleted clusters.")
 
 
 def test_9999_vcd_cse_system_stop(vcd_sys_admin):
